class GameController < ApplicationController
  layout 'default'
  before_filter :login_required
  before_filter :update, :except => [:ajax_select_moon,:ajax_get_reamaining_minutes]

  # Alias for galaxy
  def index
    galaxy
    render_action 'galaxy'
  end

  # Displays the moowaon galaxy
  def galaxy
    @player = @session[:user].player
    @all_moons = Moon.find(:all)
    @selected_moon = @session[:moon] || @player.home_moon
    seconds_to_next_round = (@player.last_update_on + Game.round_length) - Time.now
    @minutes_remaining = (seconds_to_next_round/60).to_i.to_s.rjust(2,"0")
    # I hate RadRails syntax highlighting
    @seconds_remaining = (seconds_to_next_round%60).to_i.to_s.rjust(2,"0")
  end

  # Selects a moon (this is now done with ajax calling: ajax_select_moon)
  def select_moon
    @player = @session[:user].player
  	@session[:moon] = Moon.find(params[:id]) if @player.owns(Moon.find(params[:id]))
  	redirect_to_index and return
  end

 # Moon overview
  def moon
    @player = @session[:user].player
    params[:id] ||= @player.home_moon.id
    @moon = Moon.find(params[:id])
    redirect_to_index("You don't own this moon!") and return unless @player.owns(@moon)
    @origin = @session[:moon]
    @fleet = @origin.fleet
  end
  
  # Interface for the player view
  def player
    redirect_to_index("You didn't specify a player") and return if params[:id].nil?
    @player = Player.find(params[:id])
  end
  
  # Changes the production on a moon to the posted product
  def change_product
    if @request.post?
      @moon = Moon.find(params[:id])
      @moon.cancel_production
      redirect_to_index("You don't own this moon!") and return unless @session[:user].player.owns(@moon)
      redirect_to_index("You don't have enough money!") and return unless @session[:user].player.money >= Spaceship.get_price(params[:product])*params[:amount].to_i.abs
      @moon.product = params[:product]
      @moon.amount = params[:amount]
      player = @session[:user].player
      player.money -= Spaceship.get_price(@moon.product) * @moon.amount
      player.save
      case @moon.product
      when 'satellite'
        @moon.rounds_left = 2
      when 'money'
        @moon.rounds_left = 1
      else
        @moon.rounds_left = @moon.rounds_needed_to_produce_ship
      end
      if @moon.save
        flash[:notice] = "Changed production on #{@moon.name}" 
      else
        flash[:notice] = "Could not change production on #{@moon.name}" 
      end
    end
    redirect_to_index and return
  end

  # Cancel the production on a moon
  def cancel_production
    moon = Moon.find(params[:id])
    redirect_to_index("You don't own this moon!") and return unless @session[:user].player.owns(moon)
    moon.cancel_production
    redirect_to_index("Canceled production on #{moon.name}") and return
  end
  
  # Sends the posted money to another player
  def send_money
    if @request.post?
      reciever = Player.find(params[:id])
      sender = @session[:user].player
      money = params[:amount].to_i.abs
      redirect_to_index("You don't have that much money!") and return if money > sender.money
      redirect_to_index("Trying to send no money? No money has been sent!") and return if money.zero?
      reciever.money += money
      sender.money -= money
      reciever.add_to_log "You got #{money} money from #{sender.user.login}"
      sender.add_to_log "You send #{money} money to #{reciever.user.login}"
      reciever.save
      sender.save
      redirect_to_index("You send #{money} money to #{reciever.user.login}") and return
    else
      retdirect_to_index and return
    end
  end

  # Inteferface to attack another moon
  def attack
    redirect_to_index("You didn't specify a moon!") and return if params[:id].nil?
    @player = @session[:user].player
    @moon = Moon.find(params[:id])
    @origin = @session[:moon]
    @origin.create_fleet # Create new fleet on the moon if there's none yet
    @fleet = @origin.fleet
    
    redirect_to_index("You can't attack yourself!") and return if @player.owns(@moon)
  end

  # Sends the posted fleet to another moon
  def send_fleet
    redirect_to_index and return unless @request.post?
    
    @player = @session[:user].player
    @fleet = Fleet.new(params[:fleet])
    @fleet.departure_on = @player.last_update_on
    
    @origin = Moon.find(@fleet.starting_moon)
    @destination = Moon.find(@fleet.destination_moon)
    
    @fleet.arrival_on = @player.last_update_on + @destination.time_to_reach_from(@origin)*Game.round_length
    
    if @fleet.total_strength > @origin.fleet.total_strength ||
       @fleet.weak_ships > @origin.fleet.weak_ships ||
       @fleet.medium_ships > @origin.fleet.medium_ships ||
       @fleet.strong_ships > @origin.fleet.strong_ships
      flash[:notice] = "You can't send more ships than you have!"
    elsif @fleet.total_strength.zero?
      flash[:notice] = "Trying to send no ships, hu? Well... no ships have been sent!"
    else
      @origin.fleet.substract(@fleet)
      @origin.fleet.save
      @player.fleets << @fleet
      if @fleet.save
        flash[:notice] = "You sent your fleet to #{@destination.name}"
      else
        flash[:notice] = "There was an error while sending the fleet."
      end
    end
    redirect_to_index and return
  end
  
  # Ajax funtion to select a moon
  def ajax_select_moon
    @player = @session[:user].player
    @old_moon_id = @session[:moon].id
    @new_moon_id = params[:id]
  	@session[:moon] = Moon.find(params[:id]) if @player.owns(Moon.find(params[:id]))
  	render :layout => false
  end
  
  # Ajax function that returns the minutes till the next round
  def ajax_get_remaining_minutes
    @player = @session[:user].player
    seconds_to_next_round = (@player.last_update_on + Game.round_length) - Time.now
    @minutes_remaining = (seconds_to_next_round/60).to_i.to_s.rjust(2,"0")
    # I hate RadRails syntax highlighting
    @seconds_remaining = (seconds_to_next_round%60).to_i.to_s.rjust(2,"0")
    render_text "#{@minutes_remaining}:#{@seconds_remaining} minutes"
  end
  
  # Ajax function to update the interface periodically
  def ajax_update
    @player = @session[:user].player
    seconds_to_next_round = (@player.last_update_on + Game.round_length) - Time.now
    minutes_remaining = (seconds_to_next_round/60).to_i.to_s.rjust(2,"0")
    # I hate RadRails syntax highlighting
    seconds_remaining = (seconds_to_next_round%60).to_i.to_s.rjust(2,"0")
    @countdown = "#{minutes_remaining}:#{seconds_remaining} minutes"
    @money = @player.money
    @level = @player.level
    galaxy
    render :layout => false
  end
  
  #Ajax wrapper funtion for the moon overview
  def ajax_moon
    moon
    render :layout => false
  end
  
  #Ajax wrapper funtion for the galaxy view
  def ajax_galaxy
    galaxy
    render :layout => false
  end
  
  #Ajax wrapper function for the attack view
  def ajax_attack
    attack
    render :layout => false
  end
  
  #Ajax wrapper funtion for the log
  def ajax_log
    log
    render :layout => false
  end

  # Display a log of what happened
  def log
    @player = @session[:user].player
    @events = @player.log_events.sort { |b,a| a.created_on <=> b.created_on } [0..40] # Sort events
    (@player.log_events - @events).each { |e| e.destroy } # Delete old events
  end
  
  # This is were all events are handled, this function gets called everytime you load the site
  def update
    return true if @session[:user].nil? # If there's no user, then dont run update and just say the filter passed
    player = @session[:user].player
    
    # Create a new player for the user if there's none
    if player.nil?
      @session[:user].player = Player.new
      player = @session[:user].player
      m = Moon.make
      m.home_moon = 'true'
      m.size = '2'
      m.fleet.weak_ships,m.fleet.medium_ships,m.fleet.strong_ships = 5,5,5
      player.moons << m
      player.save
      m.fleet.save
      m.save
      Galaxy.add_moons(3) # Add new moons
    end
    
    @session[:moon] ||= player.home_moon # Initialize session moon
    
    if player.rounds_passed > 0
      player.collect_taxes
      player.run_production
      player.run_fights
    
      player.last_update_on += player.rounds_passed*Game.round_length
      player.save
    
      # Run fights for attacking players
      player.moons.each do |m|
        Fleet.find(:all, :conditions => ["moving = 'true' AND destination_moon = ? AND arrival_on < NOW()",m.id]).map { |f| f.player} .uniq.each { |p| p.run_fights }
      end
    end
    true
  end

end
