class Fleet < ActiveRecord::Base
  belongs_to :moon
  belongs_to :player
  validates_numericality_of :weak_ships
  validates_numericality_of :medium_ships
  validates_numericality_of :strong_ships
  
  # The strength of the fleet
  def total_strength
    self.weak_ships + self.medium_ships*2 + self.strong_ships*3
  end
  
  # This fleet wins against the other fleet?
  def wins_against(fleet)
    self.total_strength > fleet.total_strength
  end
  
  # Shirnks the fleet to a certain strength (not very efficient yet)
  def shrink_to_strength(strength)
    until self.total_strength <= strength
      case rand(3)
      when 0:
        self.weak_ships -= 1 if self.weak_ships > 0
      when 1:
        self.medium_ships -= 1 if self.medium_ships > 0
      when 2:
        self.strong_ships -= 1 if self.strong_ships > 0
      end
    end
  end
  
  # Merges two fleets
  def merge(fleet)
    self.+(fleet)
    fleet.destroy
    self.save
  end
  
  # Stops a moving fleet
  def stop
    self.moving = 'false'
    self.arrival_on,self.departure_on,self.starting_moon,self.destination_moon = nil
  end
  
  # Fight the fleet at the destination
  def fight
    destination = Moon.find(self.destination_moon)
    destination.create_fleet # Create a new fleet if theres none on the destination
    enemy = destination.player
    if self.player.owns(destination) # The player owns the destination, then merge the fleets
      destination.fleet.merge(self)
      self.player.add_to_log("Your fleet reached #{destination.name} and is now deployed there.")
    else # The player doesn't own the destination: FIGHT! ^^
      Fight.new(destination,self)
    end
  end
  
  # Is the fleet moving?
  def moving?
    self.moving == 'true'
  end
  
  # Did the fleet arrive at its destination?
  def arrived?
    self.moving? && self.arrival_on < Time.now
  end
  
  # Add ships from another fleet
  def +(other)
    self.weak_ships += other.weak_ships
    self.medium_ships += other.medium_ships
    self.strong_ships += other.strong_ships
  end

  # Take ships away from this fleet
  def substract(other)
    self.weak_ships -= other.weak_ships
    self.medium_ships -= other.medium_ships
    self.strong_ships -= other.strong_ships
  end
  
  protected
  
  # Validates the fleet (for being saved into the database)
  def validate
    errors.add(:weak_ships,'can\'t be negativ') if self.weak_ships < 0
    errors.add(:medium_ships,'can\'t be negativ') if self.medium_ships < 0
    errors.add(:strong_ships,'can\'t be negativ') if self.strong_ships < 0
    
    if self.moving == 'true'
      errors.add(:fleet,'has to contain at least one ship') if total_strength.zero?
    end
  end
end
