class Player < ActiveRecord::Base
  has_many :moons
  has_many :log_events
  has_many :fleets
  belongs_to :user
  
  # Find the home moon of the user
  def home_moon
    Moon.find(:first, :conditions => ["home_moon = 'true' AND player_id = ?",id])
  end
  
  # Returns true if a user owns a moon
  def owns(moon)
  	moons.include?(moon)
  end

  # Calculate the level of the player
  def level
    return 0 if moons.empty?
    strength = 0
    moons.each do |m|
      strength += m.fleet.total_strength unless m.fleet.nil?
      strength += m.satellite_count * 2
    end
    (strength/moons.size).round
  end
  #
  
  # Returns the rounds that passed since the last login of the player
  def rounds_passed
    self.last_update_on = Time.now if self.last_update_on.nil?
    ((Time.now-self.last_update_on)/Game.round_length).to_i
    #
  end
  
  # Let all the fight take place for the player
  def run_fights
    self.fleets.each do |f|
      if f.arrived?
        f.fight
      end
    end
  end
  
  # Let all production take place on the player's moons
  def run_production
    self.moons.each do |m|
      unless m.product == 'money'
        amount = [m.amount, m.capacity_for_passed_rounds ].min
        product = LogEvent.pluralize( Spaceship.get_name(m.product) , amount)
        m.produce_ships( amount )
        self.add_to_log("Produced #{amount} #{LogEvent.pluralize(product,amount)} on #{m.name}") unless amount.zero?
        m.save
      end
    end
  end
  
  # Let the player collect all the taxes from his/her moon
  def collect_taxes
    collected_money = 0
    self.moons.each do |m|
      if m.product == 'money'
        if m.last_update_on.nil?
         m.last_update_on = m.player.last_update_on
        end
        collected_money += m.taxes*m.rounds_passed
        m.last_update_on += Game.round_length*m.rounds_passed
        m.save
      end
    end
    self.money += collected_money
    add_to_log("You got #{collected_money} money from #{self.moons.size} moons for #{self.rounds_passed} #{LogEvent.pluralize('round',self.rounds_passed)} passed!") unless collected_money.zero?
    collected_money
  end
  
  # Adds a messaage to the players log
  def add_to_log(msg)
    self.log_events << LogEvent.create(msg)
  end
end
