require File.dirname(__FILE__) + '/../test_helper'
require 'game_controller'

# Re-raise errors caught by the controller.
class GameController; def rescue_action(e) raise e end; end

class GameControllerTest < Test::Unit::TestCase

  fixtures :users,:players,:moons,:fleets
  	
  def setup
    @controller = GameController.new
    @request    = ActionController::TestRequest.new
    @request.host = "localhost"
    @response   = ActionController::TestResponse.new
  end

  def test_restriced_access
    @request.session[:user] = nil
    get :index
    assert_redirect_url "http://localhost/login/login"
  end
  
  def test_access
    @request.session[:user] = User.find(1)
    get :index
    assert_success
  end
  
  def test_select_moon
    @request.session[:user] = User.find(1)

    get :select_moon, :id => 6
    assert_equal Moon.find(6),@request.session[:moon]
    
    get :select_moon, :id => 3
    assert_equal Moon.find(6),@request.session[:moon]
  end
  
  def test_moon
    @request.session[:user] = User.find(1)
    
    get :moon
    assert_success
    get :moon, :id => 6
    assert_success
    get :moon, :id => 3
    assert_redirect_url "http://localhost/game/index"
  end
  
  def test_player
    @request.session[:user] = User.find(1)
    get :player
    assert_redirect_url "http://localhost/game"
    get :player, :id => 2
    assert_success    
  end
  
  def test_send_money
    @request.session[:user] = User.find(1)
    
    old_money_sender = User.find(1).player.money
    old_money_reciever = User.find(2).player.money
    
    post :send_money, :id => 2, :amount => 100
    assert_redirect
    assert_equal old_money_sender-100,User.find(1).player.money
    assert_equal old_money_reciever+100,User.find(2).player.money
    
    old_money_sender = User.find(1).player.money
    old_money_reciever = User.find(2).player.money
    
    post :send_money, :id => 2, :amount => 100000000000
    assert_redirect_url "http://localhost/game/index"
    assert_equal old_money_sender,User.find(1).player.money
    assert_equal old_money_reciever,User.find(2).player.money      
  end
  
  def test_change_product
    @request.session[:user] = User.find(1)
    
    post :change_product, :id => 6, :product => '1', :amount => 1
    assert_equal Moon.find(6).product,'1'
    assert_equal Moon.find(6).amount,1
    
    post :change_product, :id => 3, :product => '1', :amount => 1
    assert_equal "You don't own this moon!",flash[:notice]
    
    get :change_product
    assert_redirect_url "http://localhost/game"
  end
  
  def test_attack
    @request.session[:user] = User.find(1)
    get :attack
    assert_redirect_url "http://localhost/game"
    assert_equal "You didn't specify a moon!",flash[:notice]
    
    get :attack, :id => 1
    assert_redirect_url "http://localhost/game/index"
    assert_equal "You can't attack yourself!",flash[:notice]
    
    get :attack, :id => 3
    assert_success
  end
  
  def test_send_fleet
    @request.session[:user] = User.find(1)
    
    get :send_fleet
    assert_redirect_url "http://localhost/game"
    
    post :send_fleet, :fleet => { :weak_ships => 2, :medium_ships => 2, :strong_ships => 2, :moving => "true", :starting_moon => 6, :destination_moon => 3 }
    assert_equal "You sent your fleet to player_2_home_moon",flash[:notice]
    
    post :send_fleet, :fleet => { :weak_ships => 0, :medium_ships => 0, :strong_ships => 0, :moving => "true", :starting_moon => 6, :destination_moon => 3 }
    assert_equal "Trying to send no ships, hu? Well... no ships have been sent!",flash[:notice]
    
    post :send_fleet, :fleet => { :weak_ships => 55, :medium_ships => 33, :strong_ships => 11, :moving => "true", :starting_moon => 6, :destination_moon => 3 }
    assert_equal "You can't send more ships than you have!",flash[:notice]
  end
  
  def test_log
    @request.session[:user] = User.find(1)
    #get :log
    #assert_success
  end
  
end
